package org.jabber.applet;
import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Hashtable;
import java.net.URL;
import org.jabber.applet.communication.*;
import org.jabber.applet.awt.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
* This class is used to communicate with groupchat sessions.
* 
* @author David Scott
*/
class GroupChat {
    /**
     * My resource identifier for this groupchat
     */
    String myResource="";

    /**
     * Groupchat room name
     */
    String room = "";

    /**
     * Server handling the groupchat
     */
    String server = "";

    /**
     * My nickname for this groupchat
     */
    String nick = "";

    /**
     * Secret/password for this groupchat
     */
    String secret = "";

    /**
     * Nicknames associated with all resources in this groupchat
     */
    Hashtable resourceNicks = new Hashtable();

    /**
     * Initialize a groupchat room.
     * @param room name for this groupchat
     * @param server handling this groupchat
     * @param nick name I'm using in this groupchat
     * @param secret password for this groupchat
     */
    GroupChat(String room, String server, String nick, String secret) {
        this.room = room;
        this.server = server;
        this.nick = nick;
        this.secret = secret;
    }

    /**
     * Join a groupchat room.
     * Use the constructor set room name and server to join a groupchat room.
     * @param moJabberComm connection to server
     */
    public void join(JabberComm moJabberComm) {
        if (moJabberComm.groupChatJoin("GroupChatJoin", room, server) != Constants.SEND_SUCCEEDED)
            System.out.println("Trouble with groupChatJoin");
    }

    /**
     * Set my nickname for this room.
     * Use the constructor set room name, server, secret and nick to set my nick in this room.
     * @param moJabberComm connection to server
     */
    public void nick(JabberComm moJabberComm) {
        int i = 1;      // create 3 possible nicks for entranct into room
        String nicks = nick + "," + nick + i + "," + nick + (i+1);  // setup for multiple nick possibilities
        if (moJabberComm.groupChatNick("GroupChatNick", room, server, nicks, secret, false) != Constants.SEND_SUCCEEDED)
            System.out.println("Trouble with groupChatNick");
    }

    /**
     * Officially announce my presence/join the groupchat room.
     * Use the constructor set room name, server and input resource to inform
     * server of my presence.
     * @param moJabberComm connection to server
     * @param Resource associated with the presence setting.
     */
    public void presence(JabberComm moJabberComm, String Resource) {
        myResource = Resource;
        if (moJabberComm.groupChatEnter("GroupChatEnter",room, server, myResource, nick) != Constants.SEND_SUCCEEDED)
            System.out.println("problem joining chatroom");
    }

    /**
     * Leave the groupchat room.
     * Use the constructor set room name and server and my resource to inform groupchat I'm leaving.
     * @param moJabberComm connection to server
     */
    public void leave(JabberComm moJabberComm) {
        if (moJabberComm.groupChatLeave(room, server, myResource) != Constants.SEND_SUCCEEDED)
            System.out.println("problem leaving chatroom");
    }

    /**
     * Get nickname associated with a resource.
     * @param sResource associated with a nickname
     */
    public String getNick(String sResource) {
        return((String)resourceNicks.get(sResource));
    }

    /**
     * Get my resource
     */
    public String getMyResource() {
        return(myResource);
    }

    /**
     * User leaving chatroom
     */
    public void userLeave(String sResource) {
        // resourceNicks.put(sResource, sNick);
    }

    /**
     * New user joining groupchat room.
     * @param sResource of new member
     * @param sNick of new member
     */
    public void userJoin(String sResource, String sNick) {
        if (sResource.length() == 0) {
            return;
        }
        System.out.println("Resource: " + sResource);
        System.out.println("Nick: " + sNick);
        if (sNick.equals(nick)) {
            myResource = sResource;
        }
        resourceNicks.put(sResource, sNick);
    }

    /**
     * I'm changing my status in this groupchat
     * @param moJabberComm connection to server
     * @param sStatus new status
     */
    public void presenceChange(JabberComm moJabberComm, String sStatus) {
        if (moJabberComm.sendPresence(room + "@" + server, "", "", sStatus, "1", sStatus) != Constants.SEND_SUCCEEDED)
            System.out.println("problem sending presence");
    }
}
