package org.jabber.applet;
import java.awt.*;
import java.awt.datatransfer.*;
import java.applet.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Enumeration;

import java.net.URL;
import java.net.MalformedURLException;
import org.jabber.applet.communication.*;
import org.jabber.applet.awt.*;
import org.jabber.applet.connection.*;
import org.jabber.applet.tools.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * Main Roster window display
 * This class provides 2 pages of user interface
 * <br>(1) logon page (enter username, password and server) +
 * <br>(2) After succesful logon displays the users roster list
 * <br><br>
 * After logon (and display of the roster list) the user can
 * initiate/receive messages, add/remove users from his/her roster,,,,
 * 
 * <br>Parameters input to applet:
 * <br><b>xmlhostname</b> - jabber server hosting the applet (connection name)
 *  <br>        Special cases:
 *  <br>             if = null user inputs xmlhostname
 *  <br>             if = "-host-" applet uses getHost() function for this value
 * <br><b>toolbar</b>   - set to "0" when toolbar should NOT be displayed
 * <br><b>defaultColor</b> - background color (default=gray)
 * <br><b>userReg</b>   - url to be launched when user presses -new user- button from logon page.
 * <br><b>help</b>      - url to be launched if a help page is available.
 * <br><i>note: userReg param when set = 'none' causes the applet to NOT display the corresponding button.
 * <br><br>note: in the future could possibly use Resource Bundles for these parameters</i>
 * 
 * @author David Scott
 */
public class JabberApplet extends Applet implements Config, IJabberClient {
    /**
     * Applet release version number formated (a.b.c.d)
     * a = major release number
     * b = minor release number
     * c = fix release number
     * d = qa development release number
     */
    public String VERSION_NUMBER = "1.1.0.36";
/*BISCHFF*/ public JabberApplet a;	
    /**
     * Applet state (keep static due to reload)
     * 1 = stop, 3 = destroy, 4 = init
     */
    public static int appletState = 3;

    /**
     * Max number of 1/2 second tries to start group chat.
     */
    public static int MAX_GROUP_CHAT_TRIES = 120;

    /**
     * Groupchat server if it exists (returned via agents packet)
     */
    String grpChatServer = "";

    /**
     * Track all groupchats
     */
    Hashtable groupChats = new Hashtable();

    /**
     * Class to initiate, track and communicate with chat window sessions
     * note: keep static for applet reloads
     */
    public static ChatWindows Chats;

    /**
     * Debug flag when set dumps xml input/output to java console
     */
    private boolean bDebug = true;

    /**
     * Special netscape flag required due to improper sequence of init(), start(), stop(), destroy()
     */
    private static boolean bNetscapeSpecial = false;

    /**
     * Connection to Jabber server
     * note: keep public static for applet reload 
     */
    public static JabberComm moJabberComm = null;

    /**
     * Test Jabber Server connection.
     * If user isn't actively jabbering the applet fails to be informed of
     * a server disconnect, therefore periodically a null presence is sent out.
     * note: keep static for applet reload
     */
    public static TestConnect testConnect = null; // test jabber server connection

    /**
     * Vector of users that have 
     * (1) sent an unsubscribe presence packet
     * (2) we've displayed the 'User B initiated unsubscribe' msg
     * and (3) haven't received the unsubscribed presence packet
     */
    private Vector vUnsubscribers = new Vector();

    /**
     * Screen offset for chat windows used to place chile
     * windows cascading down/left so they're all seen.
     */
    private int nDlgOffsetTop = 0;

    /**
     * Screen offset for chat windows used to place chile
     * windows cascading down/left so they're all seen.
     */
    private int nDlgOffsetLeft = 0;

    /**
     * Roster List displayed
     * Note/Warning: Must be PUBLIC to work with browser
     */
    private JIDList zList = new JIDList();

    /**
     * Password textbox
     */
    private String txtPassword = "";

    /**
     * Allow user to manually logon?
     */
    private boolean autoLogonOnly = false;

    /**
      * JID describing signed on user
      */
    public JID jidMe = new JID("@");


    /**
     * Colors input by client
     */
    public Color clientColor;

    /**
     * Agents supported by the jabber server logged onto.
     * NOTE: don't forget to clear this when logoff *****
     */
    Hashtable agents = new Hashtable();

    /**
     * Convert XML special chars to standard ASCII
     */
    private StringCleanup moStringCleanup = new StringCleanup();

    /**
     * Allow applet to run in an application.
     */
    private Properties props;

    private URL pollingURL;

    private String streamID = "";

    // -----------------------------------------------------------------------
    // used to acquire a reference to the applet instance so the msgSend
    // dialog and others can call functions existing in this class
    // -----------------------------------------------------------------------
    private static JabberApplet _self = null;

    /**
     * Initialize the applet
     */
    public void run() {
        this.init();
    }

    /**
     * Save instance of self for use by child windows.
     */
    public JabberApplet() {
        _self = this;
        setSize(400,500);
        return;
    }

/** BISCHOFF: **/
 public static void main (String[] args) {
    JabberApplet a = new JabberApplet();
    a.init();
    Frame f = new Frame("JabberApplet");
    f.setLayout( new BorderLayout() );
    f.add (a, "Center");
    f.setSize(400,220);
    f.setVisible(true);
    // a.start();
    f.addWindowListener( new WindowControl() );
  }


/** bis hier **/


    /**
     * Get instance of applet
     * @return applet instance
     */
    public static JabberApplet getAppletInstance() {
        return _self;                  // return our instance
    }

    /**
     * Resize the applet.
     * Used via javascript (outside call) when the browser is resized.
     * note: currently not working, possibly due to the cardlayout manager.
     * @param width - page width
     * @param height - page height
     */
    public void setSize(int width, int height) {
        super.setSize(width, height);
        validate();
    }

    public void setUserPwd( String user, String pwd ) {
        if( autoLogonOnly ) {
            txtPassword = pwd;
            jidMe.setUsername(user);
            logonUser( jidMe.getUsername(), txtPassword );
        }
    }

    /**
     * Set properties (used primarily when running as an application.
     * @param props The properties to set.
     */
    public void setProperties( Properties props ) {
        this.props = props;
    }
    /**
     * Get properties.
     * @return Properties associated with this applet.
     */
    public Properties getProperties( ) {
        return this.props;
    }

    /**
     * Override getParameter( String ) to check properties if null.
     * @param name - the name of the parameter.
     * @return String parameter from the Applet or properties.
     */
    public String getParameter( String name ) {
        String ret = null;
        try {
            ret = super.getParameter( name );
        } catch( NullPointerException npe ) { }

        if( ret == null )                   // no input param?
        {                                   // get value from properties file
            Properties p = getProperties();
            if( p != null ) {
                ret = p.getProperty( name );
            }
        }
        return ret;
    }

    /**
     * Create Contacts/Roster Panel display.
     * Create the panel that will contain the roster list.
     */
    private void createContactsPanel() {
        //--------------------------------------------
        // Setup contact list panel with border layout
        // North - place toolbar
        // Center - place roster list
        //--------------------------------------------
        Component c = null;
        boolean b = !appletParamGet(getParameter("toolbar")).equals("none");
        ListRosterComponent list = new ListRosterComponent(zList);
        list.setProperties(props);
        c = list.getWrapper(b);
        add
            (c);
    }                                   // end, createContactsPanel

    /**
     * Parse the input rgb parameter.
     * Create valid RGB values (0-255).
     * Expected input format #,#,#
     * @param sinput rgb input parameter from user
     * @param rgb array [0]=red, [1]=green, [2]=blue
     */
    private void rgbInputParse(String sinput, int rgb[]) {
        boolean bPeachy = false;        // init flag = bad
        if ((sinput != null) && (sinput.length() > 0)) {
            int delim1;                 // delim1 ',' location
            int delim2;                 // delim2 ',' location
            delim1 = sinput.indexOf(",");   // find 1st delim
            delim2 = sinput.lastIndexOf(",");   // find 2nd delim
            if ((delim1 > 0) && (delim2 > delim1)) {                           // got valid delimeters
                try {
                    Integer i;          // used to convert input to int

                    // Extract the 3 integers from input
                    i = Integer.valueOf(sinput.substring(0, delim1));
                    rgb[0] = i.intValue();  // red
                    i = Integer.valueOf(sinput.substring(delim1+1, delim2));
                    rgb[1] = i.intValue();  // green
                    i = Integer.valueOf(sinput.substring(delim2+1, sinput.length()));
                    rgb[2] = i.intValue();  // blue

                    // Insure each rgb value is within legal bounds
                    if ((0 <= rgb[0]) && (rgb[0] <=  255))         // red in range?
                        if ((0 <= rgb[1]) && (rgb[1] <=  255))     // green in range?
                            if ((0 <= rgb[2]) && (rgb[2] <=  255)) // blue in range?
                                bPeachy = true;
                }                                          // end, try
                catch (java.lang.NumberFormatException e) {                       // catch invalid input numbers
                }
            }                           // end, got valid delimeters
        }
        // If we didn't get valid params then set default to gray
        if (bPeachy == false) {                               // input failed
            rgb[0] = 195;               // default red
            rgb[1] = 195;               // default green
            rgb[2] = 195;               // default blue
        }                               // end, input failed
    }                                   // end, rgbInputParse routine

    /**
     * Stop the applet.
     * In the case of Netscape set a flag indicating we've stopped and we're using Netscape.
     */
    public void stop() {
        appletState = 1; //1=stop, 3=destroy, 4=init
        if (System.getProperty("java.vendor").indexOf("Netscape") > -1)
            bNetscapeSpecial = true;    // indicate we've reached a stop() state
    }

    /**
     * Destroy the applet
     * Cleanup shop (required to insure browser refresh works 
     * without causing cpu 100% usage)
     */
    public void destroy() {
        appletState = 3;            //1=stop, 3=destroy, 4=init
        logOff();                   // logoff the jabber server
        bNetscapeSpecial = false;   // indicate normal init
    }

    /**
     * Get applet input parameter
     * @param sParam String name of parameter
     * @return value of parameter or "" if parameter doesn't exist
     */
    private String appletParamGet(String sParam) {
        if ((sParam==null) ||
                (sParam.equals("undefined")) ||
                (sParam.trim().length() == 0)) {
            return("");
        }
        return(sParam);
    }

    /**
     * Get name of server applet is communicating with.
     * @return name of server applet should communicate with
     */
    private String getServer() {
        String serverName = appletParamGet(getParameter("xmlhostname"));
        if (serverName.length() == 0) {
            try {
                serverName = getCodeBase().getHost();
            } catch( NullPointerException npe ) {
                serverName = "";
            }
        }
        return serverName;
    }

    /**
     * support method for gettting host name.
     * @return name of server applet should initially connect to
     */
    private String xmlHost() {
        //--------------------------------
        // Server/host setup
        // Setup the jabber connection string depending on host input:
        // host = "host" connect using same value as getCodeBase().getHost()
        // host = ??? connect using input value
        // host = null clear value for user input
        //--------------------------------
        String sXMLHostName = appletParamGet(getParameter("xmlhostname"));
        if( sXMLHostName.length() > 0 && sXMLHostName.equals("-host-")) {
            try {
                sXMLHostName = getCodeBase().getHost();
            } catch( NullPointerException npe ) {
                // running in application, let user enter.
            }
        }
        return sXMLHostName;
    }

    /**
     * Process the applet input parameters (color, username, password, xml host name)
     * Returns global params:
     * clientColor - background color for all windows
     * jidMe - username/server of currently logged in user
     * txtPassword - users password
     * bDebug - debug output setting
     */
    private void appletParamsProcess() {
        //--------------------------------
        // Create client colors
        //--------------------------------
        int rgb[] = new int[3];         // red/green/blue
        // convert string to array
        String sColors = getParameter("defaultColor");
        rgbInputParse(sColors, rgb);
        // create client color
        clientColor = new java.awt.Color(rgb[0],rgb[1],rgb[2]);

        //--------------------------------
        // Username
        //--------------------------------
        String tmp = appletParamGet(getParameter("user"));
        if (tmp == null || tmp.length() == 0)
            tmp = "";
        jidMe.setUsername(tmp);
        jidMe.setServer(getServer());

        //--------------------------------
        // Nickname
        //--------------------------------
        tmp = appletParamGet(getParameter("nick"));
        if (tmp == null || tmp.length() == 0)
            jidMe.setNick(jidMe.getUsername());
        else
            jidMe.setNick(tmp.trim());

        //--------------------------------
        // Password
        //--------------------------------
        tmp = appletParamGet(getParameter("pwd"));
        if (tmp.length() != 0)
            txtPassword = tmp;

        //--------------------------------
        // autoLogonOnly for input.
        //--------------------------------
        tmp = appletParamGet(getParameter("autoLogonOnly"));
        if( tmp != null && tmp.equalsIgnoreCase("TRUE") )
            autoLogonOnly = true;

        //--------------------------------
        // pollingURL
        //--------------------------------
        tmp = appletParamGet(getParameter("pollingURL"));
        if (tmp!=null) {
            try {
                pollingURL = new URL(getCodeBase().toString()+tmp);
            } catch (/*BISCHOFF MalformedURL*/Exception e) {}
        }

        //--------------------------------
        // Debug
        //--------------------------------
        tmp = appletParamGet(getParameter("dbg"));
        if (tmp.toString().length() > 0)
            bDebug = true;
    }

    /**
     * Initialize applet.
     */
    public void init() {
        //----------------------------------------
        // appletState: 1=stop, 3=destroy, 4=init
        // if applet hasn't been destroyed then make sure it is now.
        //----------------------------------------
        if (appletState != 3)                   // if applet hasn't been destroyed then destroy it now
            destroy();
        appletState = 4;

        //----------------------------------------
        // Initialize periodic jabber server connection testing
        //----------------------------------------
        if (testConnect == null) {
            testConnect = new TestConnect();
        }

        //----------------------------------------
        // applet general features setup
        //----------------------------------------
        appletParamsProcess();
        setFont(CLIENT_FONT);

        //----------------------------------------
        // Note: Bandaid for Netscape.  Don't perform logon
        //       when we've previously done a stop()
        //       without a destroy()
        //----------------------------------------
        if (bNetscapeSpecial != true) {                               // normal init?
            // Create Jabber Comm component
            moJabberComm = new JabberComm(this);
            moJabberComm.setDebug(bDebug);

            //----------------------------------------
            // Initialize page components
            // Create a card layout:
            // (Card 1) is a GridBagLayout used for the logon
            // (Card 2) is a BorderLayout used for roster/contact list
            //----------------------------------------
            setBackground(clientColor);     // background color
            setLayout(new GridLayout(1,1));
            // chat window communications
            Chats = new ChatWindows(clientColor);
            createContactsPanel();          // contact/roster list page

            //----------------------------------------
            // If all params available attempt autologon
            //----------------------------------------
            jidMe.setStatus(Config.LOGGED_OFF, "unavailable");
            if ((jidMe.getUsername().length() > 0)
                    && (txtPassword.length() > 0)
                    && (jidMe.getServer().length() > 0))
                logonUser( jidMe.getUsername(), txtPassword );
            else {
                logonDialog();
            }
        }                               // end, normal init
    }                                   // end, init routine

    /**
     * Display a message dialog window
     * Note: Don't try getting the parent while in the child, 
     *       getparent(), some browsers freeze
     * @param sTitle title of the window
     * @param sMsg Primary message line
     */
    private void msgDialogLaunch(String sTitle, String sMsg) {
        msgDialogLaunch(sTitle, sMsg, Config.CLIENT_FONT);
    }	// end, msgDialogLaunch routine

    /**
    * Display a message dialog window
    * Note: Don't try getting the parent while in the child, 
    *       getparent(), some browsers freeze
    * @param sTitle title of the window
    * @param sMsg Primary message line
    */
    private void msgDialogLaunch(String sTitle, String sMsg, Font sFont) {
        InputDialog oDialog = new InputDialog(this, clientColor, sFont, sTitle);

        int nloc[] = new int[2];
        childWindowPlacement( nloc, 400, 300 );     // determine screen location of next window
        oDialog.setLocation(nloc[0], nloc[1]);		// display dialog cascading down
        oDialog.msgboxOK(sMsg);
    }	// end, msgDialogLaunch routine

    /**
     * Send input message to the specified user/JID
     * note: this method is used by chat windows to send messages
     * @param jidTo jabber ID of user we're sending msg to
     * @param sendMe msg we're sending to the user
     */
    public void msgSend(JID jidTo, String sendMe) {
        /* If we are already chating with them then we should set the type
        * to "chat" so that they see them in a chat window.
        * And if they are offline/NA/XNA the message will be saved for them.
        */
        boolean bfoundChats = Chats.find(jidTo);
        String type = "";
        Object bfoundGroupChats = groupChats.get(jidTo.getUsername());
        if (bfoundChats) {
            if (bfoundGroupChats != null) {
                // Do Nothing
            } else {
                type ="chat";
                // Debug Message
                // System.out.println(jidTo.getJID() + " is " + Chats.statusGet(jidTo));
                if (!Chats.statusGet(jidTo).equalsIgnoreCase("online")) {
                    type = "";
                }
            }
        }
        if (moJabberComm.sendMsg(jidTo.getUsername() + "@" + jidTo.getServer(), type, /*subject*/"", sendMe, /*thread*/"" ) != Constants.SEND_SUCCEEDED) {
            msgDialogLaunch( PROBLEM_ERROR, MSG_NOT_SENT );
        }
    }

    /**
     * Notify user we are starting our reply.  
     * note: This is currently only supported in JabberIM
     * @param jidTo jabber ID of user we're sending msg to
     */
    /*
    public void replyStarted(JID jidTo)
    {
    String sJID = jidTo.getUsername() + "@" + jidTo.getServer() + "/" + jidTo.getResource();
       if (moJabberComm.sendMsgReplyStarted(sJID, "is replying...") != Constants.SEND_SUCCEEDED)
           msgDialogLaunch( PROBLEM_ERROR, MSG_NOT_SENT );
    }
    */
    /**
     * Close chat window corresponding to input JID
     * If the chat window is a groupchat must also
     * a) inform server we're leaving
     * b) remove the groupchat roster from roster list
     * @param jidTo jabber ID of user closing window
     */
    public void chatWindowClosing(JID jidTo) {
        Object obj = groupChats.get(jidTo.getUsername());
        if (obj != null)                // groupchat window?
        {                               // leave the room and update roster
            ((GroupChat)obj).leave(moJabberComm);
            groupChats.remove(jidTo.getUsername());
            zList.groupRemove(jidTo.getUsername());
        }                               // end, groupchat window?
        Chats.closing(jidTo);
    }

    /**
     * Groupchat being used as roster?
     * Determine from the jid if this is a special
     * groupchat jid being used as a roster list.
     * @param jid being tested for special case processing
     * @return true when input jid/roomname starts with 'special' groupchat indication.
     */
    private boolean bUsingGroupChatAsRoster(JID jid) {
        if (jid.getJID().indexOf(".") == 0)     // 'special' groupchat indication?
            return( true );
        else
            return( false );
    }

    /**
     * User has changed his/her presence, 
     * must inform all active groupchats of change.
     * @param sNewPresence users current status
     */
    public void groupChatPresence(String sNewPresence) {
        Enumeration e = groupChats.keys();
        while (e.hasMoreElements()) {
            Object obj = groupChats.get(e.nextElement());
            ((GroupChat)obj).presenceChange(moJabberComm, sNewPresence);
        }
    }

    /**
     * User desires to join/start a groupchat.
     * Prompt user for name of room to join.
     */
    public void groupChatPrompt() {
        groupChatPrompt("");
    }

    /**
     * User desires to join/start a named groupchat.
     * Prompt user for name of room to join.
     * @param sRoom name of groupchat room to join.
     */
    public void groupChatPrompt(String sRoom) {
        if (grpChatServer.length() == 0) {
            msgDialogLaunch( "Group Chat not supported", "The Group Chat module has not been installed for the " + jidMe.getServer() + " Jabber Server.");
            return;
        }

        // Get New chat room
        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Enter Chat Room");
        oInput.addQuestion("Room Name (example: Shopping):", "Room", sRoom, true,false,":@ \"<>\'&" );
        oInput.addQuestion("Nickname:", "Nick", jidMe.getNick());
        //oInput.addQuestion("Secret:");
        oInput.addButton("OK", "groupChatCreate");
        oInput.setDefaultButton("OK");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    /**
     * Create/Enter the groupchat room described.
     * @param XML string describing the room to join, nickname and password.
     */
    public void groupChatCreate(String XML) {
        MiniXMLParser oParser = new MiniXMLParser();
        String sRoom = oParser.parseTag(XML, "Room");
        String sNick = oParser.parseTag(XML, "Nick");
        //String sSecret = oParser.parseTag(XML, "Secret");
        String sSecret = "";
        if ((sNick.trim().length() == 0) || (sRoom.trim().length() ==0))
            msgDialogLaunch( "Invalid Entry", "Joining a groupchat requires both Room and Nickname inputs" );
        else
            groupChatEnter(sRoom, sNick, sSecret);
    }

    /**
     * Create/Enter the groupchat room described.
     * note: primarily used via external javascript calls
     *       to enter a chat room.
     * @param room to join/enter.
     */
    public void groupChatEnter(String room) {
        groupChatEnter(room, jidMe.getNick(), "");
    }

    /**
     * Create/Enter the groupchat room described.
     * note: primarily used via external javascript calls
     *       to enter a chat room.
     * @param room to join/enter.
     * @param nick desired for this groupchat
     * @param secret password to enter this room
     */
    public void groupChatEnter(String room, String nick, String secret) {
        boolean interrupted = false;

        // set a max time so we don't stack up
        // a bunch of sleeping threads.
        // 120 will be a total of 1 minute.
        int i = MAX_GROUP_CHAT_TRIES;

        // if the grpChatServer hasn't been initialized, wait.
        while( i > 0 && grpChatServer.length() == 0 && !interrupted ) {
            try {
                Thread.currentThread().sleep(500);
                i--;
            } catch( Exception e ) {
                interrupted = true;
            }
        }
        GroupChat grpChat = new GroupChat(room, grpChatServer, nick, secret);
        grpChat.join(moJabberComm);
        groupChats.put(room, grpChat);
        //Need to display a new chat window!!!
        //JID grpChatHnd;
        //grpChatHnd = new JID(room + "@" + grpChatServer);
        //launchChat(grpChatHnd);
    }

    /**
     * Destroy a groupchat session.
     * @param room to destroy.
     */
    public void groupChatDestroy(String sRoom) {
        JID jid = new JID(sRoom + "@" + grpChatServer);
        chatWindowClosing(jid);
    }

    /**
     * Prompt user for logon infoDestroy a groupchat session.
     */
    public void logonDialog() {
        logonDialog( null );
    }

    /**
     * Prompt user for username, password, jabber server.
     * @param caption to display with dialog.
     */
    public void logonDialog( String caption ) {
        // if user should not be presented a logon screen,
        // get outta here.
        if( autoLogonOnly )
            return;

        InputDialog oInput = null;
        String server = getServer();
        if( server == null || server.length() == 0 ) {
            oInput = new InputDialog(this, clientColor,
                                     Config.CLIENT_FONT,
                                     "Logon" );
            if( caption != null && caption.length() > 0 ) {
                oInput.addCaption( caption, true );
            }
            String name = jidMe.getUsername();
            if( name == null ) {
                name = "";
            }
            oInput.addQuestion("Username:", "username", name, true, false,":@ \"<>\'&");
            oInput.addQuestion("Password:", "password","",true,true);
            oInput.addQuestion("Jabber Server:", "server", jidMe.getServer(),
                               true, false);
        } else {
            oInput = new InputDialog(this, clientColor,
                                     Config.CLIENT_FONT,
                                     "Logon to " + xmlHost() );
            if( caption != null && caption.length() > 0 ) {
                oInput.addCaption( caption, true );
            }
            String name = jidMe.getUsername();
            if( name == null ) {
                name = "";
            }
            oInput.addQuestion("Username:", "username", name, true, false);
            oInput.addQuestion("Password:", "password","",true,true);
            oInput.addHidden("server", server);
        }
        oInput.addButton("Log On", "logon");
        oInput.setDefaultButton("Log On");
        oInput.addButton("Cancel");
        oInput.addButton(Config.NEW_ACCOUNT,"newAccount");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    /**
     * Parse dialog input user/pwd/server and logon user.
     * @param XML containing user/pwd/server info.
     */
    public void logon(String XML) {
        MiniXMLParser oParser = new MiniXMLParser();
        String server = oParser.parseTag(XML, "server");
        jidMe.setServer(server);
        String user = oParser.parseTag(XML, "username");
        String pass = oParser.parseTag(XML, "password");
        txtPassword = pass;
        logonUser( user, pass );
    }
    /**
    *   Create New Account
    */
    public void newAccount(String XML) {
        //test xml newAccountDialog(XML);
        MiniXMLParser oParser = new MiniXMLParser();
        jidMe.setUsername(oParser.parseTag(XML, "username"));
        txtPassword = oParser.parseTag(XML, "password");
        jidMe.setServer(oParser.parseTag(XML, "server"));

        createNewAccount( jidMe.getUsername(), txtPassword, jidMe.getServer() );
    }
    public void newAccountDialogNoCaption(String caption) {
        //logOff();
        newAccountDialog(null);
    }

    public void newAccountDialog(String caption) {
        // Get the registration window
        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, Config.NEW_USER_REGISTRATION);
        if( caption != null && caption.length() > 0 ) {
            oInput.addCaption( caption );
        }
        //get user name
        String name = jidMe.getUsername();
        //get server
        String server = getServer();
        //String server = jidMe.getServer();

        if( name == null ) {
            name = "";
        }
        oInput.addQuestion(Config.USERNAME+":", "username", name, true, false);
        oInput.addQuestion(Config.PASSWORD+":", "password","",true,true);
        //if we don't have a server, let the user enter one: should not happen when code ran in an applet
        if( server == null || server.length() == 0 ) {                                   //application case
            oInput.addQuestion("Jabber Server:", "server", jidMe.getServer(),true, false);
        } else {
            oInput.addQuestion(Config.JABBER_SERVER+":", "server",server,false,false); //applet case
        }
        oInput.addButton("OK", "newAccount");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    private void setDefaultCursor(String xml) {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Compute the location of the next child window so they cascade down/right
     * @param loc[] (0) = top, (1) = left
     * @param nWidth = width of next child
     * @param nHeight = height of next child
     */
    private void childWindowPlacement( int loc[], int nWidth, int nHeight ) {
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        int nInitialLeft = d.getSize().width/2 - nWidth/2;
        int nInitialTop = d.getSize().height/2 - nHeight/2;
        if (((nInitialTop + nDlgOffsetTop + nHeight) > d.getSize().height) || ((nInitialLeft + nDlgOffsetLeft + nWidth) > d.getSize().width)) {
            nDlgOffsetTop = 0;
            nDlgOffsetLeft = 0;
        } else {
            nDlgOffsetTop += 20;
            nDlgOffsetLeft += 20;
        }
        loc[0] = nInitialLeft + nDlgOffsetLeft;
        loc[1] = nInitialTop + nDlgOffsetTop;
    }

    /**
     * Logoff jabber server.
     * Logoff the jserver and clear the roster display (and corresponding lists/vectors).
     */
    public void logOff() {
        testConnect.testing(false);
        if (moJabberComm.logoff() == Constants.SEND_FAILED)
            System.out.println("JabberApplet.logOff:connection logoff failed");
        if (moJabberComm.disconnect() == Constants.SEND_FAILED)
            System.out.println("JabberApplet.logOff:disconnect failed");
        Chats.clearAll();           // remove all chat window sessions
        zList.removeAll();          // remove all list items (groups)
        jidMe.setStatus(Config.LOGGED_OFF, "unavailable");
    }                               // end, logOff

    /**
     * Launch the specified URL in a new browser window.
     * @param sURL URL to be launched
     * @param sHeader Header to be used if the function isn't supported.
     * @param sWindow browser window to place input URL
     */
    private void launchURL(String sURL, String sHeader, String sWindow) {
        if (sURL.length() == 0)
            msgDialogLaunch( sHeader, "This function is not supported on the server hosting this jabber session.  Server hosting this session: " + jidMe.getServer() );
        else {
            URL myURL1=null;
            try {
                myURL1=new URL(getDocumentBase(), sURL);
                getAppletContext().showDocument(myURL1, sWindow);
            } catch (java.net.MalformedURLException ex) {
                ;
            }
        }
    }                                   // end, launchURL

    /**
     * Launch New Contact dialog for user to enter subscription info.
     * New Contact dialog will invoke appropriate method to complete function.
     */
    public void presenceSubscribeInfo() {
        // Show New contact dialog
        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Add Contact");
        oInput.addQuestion("Add a new contact to your list (example: jsmith@jabber.com)","JID");
        oInput.addQuestion("Nickname: (this name will appear in your contact list)", "Nick");
        oInput.addQuestion("Group to add this contact to:", "Group", Config.FRIENDS);
        oInput.addButton("OK", "presenceSubscribe");
        oInput.setDefaultButton("OK");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }                                   // end, presenceSubscribeInfo

    /**
     * Update the currently selected user
     * Update Contact dialog will invoke appropriate method to complete function.
     */
    public void presenceUpdateInfo() {
        //----------------------------------------
        // If nothing or group selected do nothing
        //----------------------------------------
        if( (zList.isJIDSelected()==false) )  // nothing selected?
        {
            msgDialogLaunch( Config.PROBLEM_ERROR, "Please select a contact prior to attempting the function." );
            return;
        }

        //----------------------------------------
        // Launch Update Contact dialog for user input.
        //----------------------------------------
        JID jid = zList.getSelectedJID();
        jid.setGroup(zList.getGroupName(jid, JID.MSK_USERNAME + JID.MSK_SERVER));

        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Edit Contact");
        oInput.addCaption("Edit this contact in your list:");
        oInput.addCaption("Jabber ID: "+jid.getJID());
        oInput.addHidden("JID", jid.getJID());
        oInput.addQuestion("Nickname: ", "Nick", jid.getNick());

        String sGroup = jid.getGroup();
        if (sGroup.length() == 0)
            sGroup = Config.FRIENDS;
        oInput.addQuestion("Group to add this contact to:", "Group", sGroup);
        oInput.addButton("OK", "presenceSubscribe");
        oInput.setDefaultButton("OK");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    /**
     * Remove the currently selected user (unsubscribe)
     * Remove Contact dialog will invoke appropriate method to complete function.
     */
    public void presenceUnsubscribeRequest() {
        //----------------------------------------
        // If nothing or group selected do nothing
        //----------------------------------------
        if ((zList.isGroupSelected()==true) || (zList.isJIDSelected()==false))          // nothing selected?
        {
            msgDialogLaunch( Config.PROBLEM_ERROR, "Please select a contact prior to attempting the function." );
            return;
        }

        //----------------------------------------
        // Launch Remove Contact dialog for user input.
        //----------------------------------------
        JID jid = zList.getSelectedJID();

        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Remove Contact");
        oInput.addCaption("Remove this contact from your list:");
        oInput.addCaption("Jabber ID: "+jid.getJID());
        oInput.addHidden("JID", jid.getJID());
        oInput.addCaption("Nickname: "+jid.getNick());
        oInput.addHidden("Nick", jid.getNick());

        oInput.addButton("OK", "presenceUnsubscribe");
        oInput.setDefaultButton("OK");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 220 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    /**
     * Launch a chat window with the input user.
     * @param jid of user to start chatn with
     */
    public void launchChat( JID jid ) {
        if(zList.getGroupName(jid, JID.MSK_USERNAME + JID.MSK_SERVER).equals(Transports)) {
            ; //do nothing
        } else {
            InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Enter Chat Room");
            if ((jid.getShow() == null) || jid.getShow().equals("")) {
                oInput.addCaption(" ");
                oInput.addCaption(jid.getNick() + " is currently offline. Messages will be stored until they come back online.");
            } else if (jid.getShow().equalsIgnoreCase("online")) {
                startChat("<JID>" + jid.getJID() + "</JID><Nick>" + jid.getNick() + "</Nick>");
                return;
            } else if (jid.getShow().equalsIgnoreCase("away")) {
                oInput.addCaption(" ");
                oInput.addCaption(jid.getNick() + " is currently Away. Messages will be displayed immediately.");
            } else if (jid.getShow().equalsIgnoreCase("xa")) {
                oInput.addCaption(" ");
                oInput.addCaption(jid.getNick() + " is currently Away. Messages will be displayed immediately.");
            } else if (jid.getShow().equalsIgnoreCase("dnd")) {
                msgDialogLaunch( "Do Not Disturb", jid.getNick() + " is currently Do Not Disturb. Messages will NOT be delivered. Please try to contact them again when their status changes." );
                return;
            }
            else {
                oInput.addCaption(" ");
                oInput.addCaption(jid.getNick() + " is currently " + jid.getShow() + ". Messages will be displayed immediately.");
            }

            oInput.addHidden("JID", jid.getJID());
            oInput.addHidden("Nick", jid.getNick());
            oInput.addButton("OK", "startChat");
            oInput.setDefaultButton("OK");
            oInput.addButton("Cancel");

            int nloc[] = new int[2];         // top/left
            childWindowPlacement( nloc, 240, 250 );
            oInput.setLocation(nloc[0], nloc[1]);
            oInput.inputbox();
        }
    }


    public void startChat(String XML) {
        JID jid = getJIDFromXML(XML);
        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 200, 200 );
        Chats.startChatn(jid.getNick(), jidMe, jid, nloc[0], nloc[1]);
    }

    /**
     * To register a new User.
     */
    public void newUserRegistration() {
        launchURL(appletParamGet(getParameter("userReg")),
                  //"New User Registration", "_self");
                  "Create New Account", "_self");
    }

    /**
     * Launch help browser.
     */
    public void launchHelp() {
        launchURL(appletParamGet(getParameter("help")),
                  "Jabber Applet Help", "_blank");
    }

    /**
     * Display Version.
     */
    public void displayVersion() {
        msgDialogLaunch( "Jabber Applet", VERSION + " " + VERSION_NUMBER );
    }

    /**
     * Display Whoami.
     */
    public void displayWhoami() {
        String host = xmlHost();
        if( host == null || host.equals("") ) {
            host = jidMe.getServer();
        }
        msgDialogLaunch( "Who Am I",
                         "In chat windows you are " + jidMe.getUsername()
                         + ".  Your Jabber ID is " + jidMe.getUsername() + "@"
                         + host);
    }

    /**
     * Toggle debuging.
     */
    public void toggleDebug() {
        bDebug = !bDebug;
        moJabberComm.setDebug(bDebug);
        if (bDebug)
            System.out.println("Debug i/o on");
        else
            System.out.println("Debug i/o off");
    }

    /**
     * Jabber Server agent InfoQuery received.
     * Save vector of each agent served by this server.
     * If presence received that matches an agent a corresponding transport/gateway
     * is displayed in the roster list.
     * @param sQuery xml information describing the agents handled by current server.
     */
    private void agentsProcessing(String sQuery) {
        Object o = zList.getElementAt(0);
        String server = "";
        String service = "";
        String name = "";       // groupchat

        MiniXMLParser oParseAgent = new MiniXMLParser();
        MiniXMLParser oParseTag = new MiniXMLParser();

        // parse agents
        String sAgent = oParseAgent.parseTag(sQuery, "agent", true);
        while (sAgent.length() > 0 ) {
            server = oParseTag.parseTag(sAgent, "jid");
            service = oParseTag.parseTag(sAgent, "service");

            // groupchat allow for parameter passing of "conferenceserver"
            grpChatServer = appletParamGet(getParameter("conferenceserver"));
            if (grpChatServer.length() == 0) {
                grpChatServer = "conference.jabber.org"; // Still Not Right
            }

            name = oParseTag.parseTag(sAgent, "name");
            if (service.equals("conference")) {
                //grpChatServer = server;         // found group chat server
                //Discovery is Broken uses deprecated iq:agents discovery
                //Hard coded Hack to make it work
            } else
                agents.put(server, service);
            sAgent = oParseAgent.parseNextTag(sQuery, "agent", true);
        }

        // done with initialization, start periodic connection testing.
        testConnect.testing(true);
        if (testConnect.getStarted() == false)
            testConnect.start();

    }

    //===================================================
    //========= Sending message to jabber server
    //==========v=v=v=v=v=v=v=v=v=v=v=v=v=v=v=v===========
    /**
     * Logon a User.
     * We've received username, password and server. Now attempt to:
     * (a) connect to the specified server
     * (b) logon/authenticate the username/password user
     * (c) set the current users status to available
     * (d) request presence status for all users in our roster.
     */
    private boolean logonUser ( String user, String pass ) {
        // Set username from logon screen
        if (user.indexOf("@") > -1) {
            user = user.substring( 0, user.indexOf("@") );
        }
        jidMe.setUsername(user);
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        // Insure valid name param
        if (jidMe.getUsername().length() == 0) {
            logonDialog();
            return false;
        }

        // Connect to jabber server
        System.out.println("Start of connect ...");
        String xmlHost = xmlHost();
        if( xmlHost.equals("") ) {
            xmlHost = jidMe.getServer();
        }
        streamID = "";
        if (moJabberComm.connect(jidMe.getServer(), xmlHost, Config.DEFAULT_CONNECT_PORT, pollingURL) != Constants.SEND_SUCCEEDED) {
            System.out.println("Connect failed ...");
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            msgDialogLaunch( Config.PROBLEM_ERROR, Config.PROBLEM_CONNECT + xmlHost);
            return false;
        }
        System.out.println("Connect succeeded ...");

        // Clearup the old roster list.
        zList.removeAll();
        zList.insertElementAt("Please wait...", 0);

        // Wait for stream id (needed for digest auth)
        while(streamID.equals("")) {
            try {
                Thread.currentThread().sleep(100);
            } catch (Exception e) {
                System.out.println(e);
            }
        }

        // Logon to server
        if (moJabberComm.logon( user, pass, Config.RESOURCE, streamID) != Constants.SEND_SUCCEEDED) {
            zList.removeAll();
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            logonDialog( Config.PROBLEM_LOGON + jidMe.getUsername() );
            return false;
        }

        // Request agent information
        if (moJabberComm.sendAgentsRequest( xmlHost, "Agents") != Constants.SEND_SUCCEEDED) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            msgDialogLaunch( Config.PROBLEM_ERROR, "Request Agents failed");
            return true;
        }

        return true;
    }                                   // end, logonUser routine

    /**
    * Create a new account. calls the createNewAccount ( String user, String pass, String server ) method
    */
    private boolean createNewAccount( String user, String pass ) {
        return createNewAccount (user,pass,jidMe.getServer());
    }

    /**
    * Create a new account.
    * We've received username, password and server. Now attempt to:
    * (a) connect to the specified server
    * (b) send registration request to the server
    */
    private boolean createNewAccount ( String user, String pass, String server ) {
        // Set username from logon screen
        if (user.indexOf("@") > -1) {
            user = user.substring( 0, user.indexOf("@") );
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        boolean invalidEntry=false;
        StringBuffer invalidText=new StringBuffer();

        // Insure valid name param
        if (user.length() == 0) {
            invalidEntry=true;
            invalidText.append(" "+Config.USERNAME);
        }

        // Insure valid pass param
        if (pass.length() == 0) {
            if (invalidEntry)
                invalidText.append(",");
            else
                invalidEntry=true;
            invalidText.append(" "+Config.PASSWORD);
        }

        // Insure valid server param
        if (server.length() == 0) {
            if (invalidEntry)
                invalidText.append(",");
            else
                invalidEntry=true;
            invalidText.append(Config.JABBER_SERVER);
        }
        if (invalidEntry) {
            newAccountDialog("Invalid fields:" + invalidText.toString());
            return false;
        }

        // Connect to jabber server: will disconnect if already connected and connect to the server
        System.out.println(" *** *** Connect to server");
        if (moJabberComm.connect(server, server, Config.DEFAULT_CONNECT_PORT, pollingURL) != Constants.SEND_SUCCEEDED) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            msgDialogLaunch( Config.PROBLEM_ERROR, Config.PROBLEM_CONNECT + jidMe.getServer());
            return false;
        }

        System.out.println(" *** *** Create new account");
        //attempt to create the new account
        if (moJabberComm.createNewAccount( user, pass, Config.RESOURCE) != Constants.SEND_SUCCEEDED) {
            zList.removeAll();
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            msgDialogLaunch( Config.PROBLEM_ERROR, Config.PROBLEM_REGISTER + jidMe.getUsername());
            return false;
        }
        return true;

    }                                   // end, createNewAccount routine

    /**
     * User rejects a subscription reqeust.  
     * Send unsubscribed note related to user subscription being rejected.
     * 
     * @param newJID New user who's subscription is being rejected.
     */
    public void presenceUnsubscribed(JID newJID) {
        if (moJabberComm.sendPresence(newJID.getJID(),"","unsubscribed","","1","") != Constants.SEND_SUCCEEDED)
            System.out.println("failed unsubscribed");
    }

    /**
     * Send "online" status for user and replace toolbar to normal state.
     */
    public void presenceSetAvail() {
        String sStatus = "Online";
        groupChatPresence(sStatus);
        if (moJabberComm.sendPresence("","","",sStatus,"1","") == Constants.SEND_SUCCEEDED)
            jidMe.setStatus(Config.AVAIL, "online");
        else
            msgDialogLaunch( Config.PROBLEM_ERROR, Config.PROBLEM_SET_STATUS );
    }

    /**
     * Send "away" status to server and replace toolbar with an away indicator.
     */
    public void presenceSetUnavail() {
        String sStatus = "Away";
        groupChatPresence(sStatus);
        if (moJabberComm.sendPresence("","","","Away","1","away") == Constants.SEND_SUCCEEDED)
            jidMe.setStatus(Config.UNAVAIL, "unavailable");
        else
            msgDialogLaunch( Config.PROBLEM_ERROR, Config.PROBLEM_SET_STATUS );
    }

    /**
     * Subscribe to a users presence (Called by InputDialog via Callback)
     * @param XML is the JID in XML format of the user being subscribed to
     */
    public void presenceSubscribe(String XML) {
        presenceSubscribe(getJIDFromXML(XML));
    }

    /**
     * Subscribe to a users presence
     * @param newItem user being subscribed to
     */
    public void presenceSubscribe(JID newJID) {
        newJID.setNick(newJID.getNick().trim());

        // Trap case where user is attempting to add self
        if (newJID.getJID().equalsIgnoreCase(jidMe.getJID())) {
            msgDialogLaunch( "Invalid Entry", "Adding yourself to the contact list is not allowed.");
            return;
        }

        // subscribe to users presence
        if (moJabberComm.sendPresence(newJID.getJID(),"","subscribe","Normal Subscription Request","1","") != Constants.SEND_SUCCEEDED)
            System.out.println("failed sending subscribed presence");
        if (moJabberComm.sendRosterSet(newJID.getJID(), newJID.getNick(), newJID.getGroup(), "subscribe") != Constants.SEND_SUCCEEDED)
            System.out.println("failed sendRosterSet");
    }

    /**
     * Add the specified user to the roster list.  
     * This is accomplished via sending the appropriate roster packet(s) to the server.
     * (Called by InputDialog via Callback)
     * @param XML is the JID in XML format of the User to be added
     */
    public void rosterAddUser(String XML) {
        rosterAddUser(getJIDFromXML(XML));
    }

    /**
     * Add the specified user to the roster list.  
     * This is accomplished via sending the appropriate roster packet(s) to the server.
     * @param newJID User to be added
     */
    public void rosterAddUser(JID newJID) {
        // send <presence type=subscribed
        if (moJabberComm.sendPresence(newJID.getJID(),"","subscribed","","1","") != Constants.SEND_SUCCEEDED)
            System.out.println("failed sending subscribed presence");

        // add user to this roster (send to server)
        if (moJabberComm.sendRosterSet(newJID.getJID(), newJID.getNick(), newJID.getGroup()) != Constants.SEND_SUCCEEDED)
            System.out.println("failed sendRosterSet");

        // send <presence type=subscribe
        if (moJabberComm.sendPresence(newJID.getJID(),"","subscribe","","1","") != Constants.SEND_SUCCEEDED)
            System.out.println("failed sending subscribe back");
    }

    /**
     * Decline a subscription request.
     * (Called by InputDialog via Callback)
     * @param XML is the JID in XML format of the user to be removed from roster list
     */
    public void declineSubscription(String XML) {
        declineSubscription(getJIDFromXML(XML));
    }

    /**
     * Decline a subscription request.
     * (Called by InputDialog via Callback)
     * @param jid user to be removed from roster list
     */
    public void declineSubscription(JID jid) {
        // Send unsubscribed presence.
        if (moJabberComm.sendPresence(
                    jid.getJID(),   // to.
                    jidMe.getJID(), // from.
                    "unsubscribed", // type.
                    "",             // status.
                    "",             // priority.
                    ""              // show.
                ) != Constants.SEND_SUCCEEDED)
            msgDialogLaunch( Config.PROBLEM_ERROR, PROBLEM_MODIFY_SUBSCRIBE );
        // Send unsubscribe to server.
        if (moJabberComm.sendUnsubscribe(jid.getJID(), jid.getNick()) != Constants.SEND_SUCCEEDED)
            msgDialogLaunch( Config.PROBLEM_ERROR, PROBLEM_MODIFY_SUBSCRIBE );
        else
            zList.jidRemove(jid,  JID.MSK_USERNAME + JID.MSK_SERVER);
    }

    /**
     * Send unsubscribe to server corresponding to input user and 
     * a) If the item is a group header then shrink/expand the group
     * b) If the item is a user then start a chat window with the selected user
     * global param used zList (roster listbox)
     * (Called by InputDialog via Callback)
     * @param XML is the JID in XML format of the user to be removed from roster list
     */
    public void presenceUnsubscribe(String XML) {
        presenceUnsubscribe(getJIDFromXML(XML));
    }

    /**
     * Send unsubscribe to server corresponding to input user and 
     * a) If the item is a group header then shrink/expand the group
     * b) If the item is a user then start a chat window with the selected user
     * global param used zList (roster listbox)
     * @param jid user to be removed from roster list
     */
    public void presenceUnsubscribe(JID jid) {
        // Send unsubscribe to server
        if (moJabberComm.sendUnsubscribe(jid.getJID(), jid.getNick()) != Constants.SEND_SUCCEEDED)
            msgDialogLaunch( Config.PROBLEM_ERROR, PROBLEM_MODIFY_SUBSCRIBE );
        else
            zList.jidRemove(jid,  JID.MSK_USERNAME + JID.MSK_SERVER);
    }


    /**
     * Send message to selected user
     */
    public void sendMessage() {
        if( zList.isJIDSelected() )
            sendMessage( (JID)zList.getSelectedItem() );
    }

    /**
     * Send message to jid user
     * @param jid of user send message to
     */
    public void sendMessage( JID jid ) {
        launchChat(jid);
    }

    /**
     * Perform any special processing due to the xtra paramter
     * @param xtra - additional information
     */
    private void processXtraParam(String xtra) {
        // If an <imgroup> tag exists in the xtra parameter we're using groupchat
        // functionality to create a dynamic roster list.  Anyone joining the IMgroup
        // is added to the roster list (as an IM type roster entry).
        MiniXMLParser oParseXtra = new MiniXMLParser();
        String sIMGroup = oParseXtra.parseTag(xtra, "imgroup", false);
        if (sIMGroup.length() > 0)
            groupChatEnter("." + sIMGroup);
    }

    //===================================================
    //========= Required iJabberClient methods
    //==========v=v=v=v=v=v=v=v=v=v=v=v=v=v=v=v===========
    /**
     * Information Query Received (Incoming Jabber infoQuery).
     * <br>(a) when there's an error with an authentication packet such as invalid username/password
     * <br>(b) when there's a response to a roster request
     * Parse the query string into roster items to be added/displayed in the roster list.
     * @param sTo JID of who the iq is bound for (me)
     * @param sFrom JID of who the iq is from
     * @param sID unique id for this iq
     * @param sType type of iq message
     * @param sQuery remainder of the infoquery string to be parsed
     * @param sError summary of problem (if any_
     * @param sCode ???
     */
    public void infoQueryReceived(String sTo, String sFrom, String sID, String sType,
                                  String sQuery, String sError, String sCode) {
        System.out.println("infoQueryReceived: To: [" + sTo + "] From: [" + sFrom + "] ID: [" + sID + "] Type: [" + sType + "] Query: [" + sQuery + "] Error: ["+sError+"] Code: ["+sCode+"]" );
        //---------------------------------
        // Process agent information
        //---------------------------------
        if (sID.equals("Agents")) {
            agentsProcessing(sQuery);

            // Request roster information
            if (moJabberComm.sendRosterRequest(Config.RESOURCE) != Constants.SEND_SUCCEEDED) {
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                msgDialogLaunch( Config.PROBLEM_ERROR, Config.MSG_SEND_ROSTER_FAIL);
                return;
            }

            // Send Available status so receive presence packets
            presenceSetAvail();

            // Xtra param processing
            String tmp = appletParamGet(getParameter("xtra"));
            if (tmp == null || tmp.length() == 0)
                ;
            else
                processXtraParam(tmp);

            return;
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

        //---------------------------------
        // Problems?, no need to continue
        //---------------------------------
        if ( sID != null && sID.equalsIgnoreCase(Config.LOGON) && sCode.equals("401")) { // error 401
            zList.removeAll();
            logonDialog( Config.PROBLEM_AUTH );
            return;
            //New account packets
        } else if ( sID != null && sID.equalsIgnoreCase(Config.CREATE_NEW_ACCOUNT)) { //this is a registration packet
            if (sCode.equals("409")) { // error 409: unauthorized username
                zList.removeAll();
                newAccountDialog( Config.PROBLEM_REGISTER );
                return;
            } else {  //no error, logon the new user
                logonUser ( jidMe.getUsername(), txtPassword );
                msgDialogLaunch( Config.NEW_USER_REGISTRATION, Config.NEW_USER_REGISTRED+jidMe.getUsername()+ "@"+ jidMe.getServer());
                return;
            }
        } else if( sType != null && sType.equalsIgnoreCase("ERROR") ) {
            // do nothing but return.
            return;
        }

        //---------------------------------
        // Cleanup input query info
        //---------------------------------
        if (sQuery == null)
            sQuery = "";
        String sQueryCleaned = moStringCleanup.Cleanup(sQuery);

        //--------------------------------------------
        // Parse infoquery
        // 1) find item, subscription and JID
        // 2) update roster with JID info
        // 3) repeat steps 1-3 for each <item> input
        //--------------------------------------------
        MiniXMLParser itemParser = new MiniXMLParser();
        String sItem;

        //-----------------------------------
        // GroupChat processing
        //-----------------------------------
        JID roomJID = new JID(sFrom);
        Object roomObj = groupChats.get(roomJID.getUsername());

        if (roomObj != null) {
            // Got response from joining a room?
            // Send nickname to that room (which will respond with <presence> of all users in that room)
            if (sID.equals("GroupChatJoin")) { // process join
                // Received response to GroupChatJoin packet recently sent.
                // Now we need to send our nickname to be used in this room.
                if (roomObj != null)
                    ((GroupChat)roomObj).nick(moJabberComm);
                return;
            }                               // end, process join

            sItem = itemParser.parseTag(sQueryCleaned, "item", true);
            MiniXMLParser elementParser = new MiniXMLParser();
            String sResource = itemParser.parseTag(sItem, "res", true);
            String sName = elementParser.parseTag(sQueryCleaned, "name", true);
            // handle case where nick instead of name
            if (sName.length() == 0) {
                sName =  elementParser.parseTag(sQueryCleaned, "nick", true);
            }
            String sJID = elementParser.parseTag(sQueryCleaned, "jid", true);

            // Got response from setting nickname?
            // OR response due to new user entering room
            // (1) send <presence> to officially enter the room
            // (2) display a new chat window for this room
            // (3) parse the <item> tags and add users to the new room/roster group.
            if (sID.equals("GroupChatNick")) {                              // process nick
                // Received response to setting nick.
                // Now send presence to the newly created/joined room
                // (after extracting the resource associated with myself).
                if (roomObj != null)        // hopefully I'm the 1st one
                    ((GroupChat)roomObj).presence(moJabberComm, sResource);

                // Starting a groupchat, open the chat window.
                int nloc[] = new int[2];         // top/left
                childWindowPlacement( nloc, 200, 200 );

                //This needs to be fixed so that a nick clash can't happen.
                //jidMe.setNick(sName);

                // if we're not using groupchat as roster start chat window
                if (bUsingGroupChatAsRoster(roomJID) == false)
                    Chats.startChatn(roomJID.getJID(), jidMe, roomJID, nloc[0], nloc[1], true);
            }

            // Add all users to the roster list
            // (using the roomname as the group)
            JID rosterJID = new JID(sJID);

            while(rosterJID.getResource().length() > 0 ) {                                // more <items> to parse?
                if ((roomObj != null) && (sName.length() > 0)) {
                    ((GroupChat)roomObj).userJoin(rosterJID.getResource(), sName);
                }

                if (sJID.length() > 0) {
                    rosterJID.setGroup(roomJID.getUsername());
                    rosterJID.setNick(sName);
                    rosterJID.setUsername(rosterJID.getNick());
                    rosterJID.setServer(roomJID.getServer());

                    // If user doesn't already exist, add to the roster display
                    if (zList.getJID(rosterJID, JID.MSK_USERNAME + JID.MSK_SERVER + JID.MSK_RESOURCE + JID.MSK_GROUP) == null) {
                        zList.jidAdd(rosterJID.getGroup(), rosterJID);
                    }
                }
                sJID = elementParser.parseNextTag(sQuery, "jid", true);
                sName = elementParser.parseNextTag(sQuery, "name", true);
                // Handle users added with nick= instead of name=
                if (sName.length()== 0) {
                    sName = elementParser.parseNextTag(sQuery, "nick", true);
                }
                rosterJID = new JID(sJID);
            }
            return;
        }    // end, process nick
        //-----------------------------------
        // end, groupchat processing
        //-----------------------------------

        //---------------------------------
        // If nothing left to parse adios.
        // 1st timer? Display howto info
        //---------------------------------
        if (sQueryCleaned.length() == 0) {
            if (sID.equals(Config.RESOURCE)) {
                zList.removeAll();
                zList.add("First Time User?");
                zList.add("1) Click User");
                zList.add("     from the Main Menu");
                zList.add("2) Click Add Contact");
                zList.add("3) Complete the form");
                zList.add("     to add a new contact");
            }
            return;
        }
        // parse Items
        sItem = itemParser.parseTag(sQueryCleaned, "item", true);
        while (sItem.length() > 0 ) {                                // more <items> to parse?
            IQItem nxtItem = new IQItem(sItem);

            //-------------------------------------
            // no username?  possible transport
            // Setup transport to look like normal JID
            // so remainder of code can process it
            //-------------------------------------
            if (nxtItem.getName().equals("")) {                                        // no username?
                if (agents.containsKey(nxtItem.getServer())) {                                   // transport?
                    nxtItem.setNick((String)agents.get(nxtItem.getServer()));
                    nxtItem.setName((String)agents.get(nxtItem.getServer()));
                    nxtItem.setGroup(Transports);
                }                                   // end, transport?
            }                                       // end, no username?

            //-------------------------------------
            // Create JID from iq item
            //-------------------------------------
            if (nxtItem.getNick().length() == 0 )   // no nickname, use JID
                nxtItem.setNick(nxtItem.getName() + "@" + nxtItem.getServer());

            if (JID.validJID(nxtItem.getName(), nxtItem.getServer()) == true) {
                JID newJID = new JID(nxtItem.getName(), nxtItem.getServer(), nxtItem.getNick(), nxtItem.getGroup());
                //-------------------------------------
                // Perform appropriate function from subscription request
                // subscription ask
                //  'remove'                    - remove member from roster list
                //  ''          ''              - send unsubscribe
                //  'none'      'subscribe'     - append <pending> status to roster display
                //-------------------------------------
                if (nxtItem.getSubscription().equals("remove")) { // subscription == remove
                    if (zList.getJID(newJID, JID.MSK_USERNAME + JID.MSK_SERVER) != null)
                        zList.jidRemove(newJID,  JID.MSK_USERNAME + JID.MSK_SERVER);
                }
                // subscription=none & ask=null
                else if ((nxtItem.getSubscription().equals("none")) && (nxtItem.getAsk().equals("")))
                    presenceUnsubscribe(newJID);
                else {                           // all other scenarios (including subscription==null)
                    try {
                        // handle case subscription-none, ask-subscribe (append 'pending' to status)
                        //if ((nxtItem.getSubscription().equals("none")) && (nxtItem.getAsk().equals("subscribe")))
                        if (nxtItem.getAsk().equals("subscribe"))
                            newJID.setStatus("Pending", "pending");

                        JID jid = zList.getJID(newJID, JID.MSK_USERNAME + JID.MSK_SERVER);
                        if (jid != null) {
                            // updating a current jid in the list, save current status for redisplay
                            newJID.setStatus(jid.getStatus(), jid.getShow());
                            zList.jidRemove(newJID,  JID.MSK_USERNAME + JID.MSK_SERVER);
                        }
                        zList.jidAdd(nxtItem.getGroup(), newJID,false);
                    } catch (RuntimeException e) { }            // invalid constuctor values
                }
            }
            sItem = itemParser.parseNextTag(sQuery, "item", true);
        }                               // more <items> to parse?
        zList.display();
    }                                   // infoQueryReceived


    /**
     * Input tag encountered that's not currently supported
     * @param sTagName unsupported tag name
     * @param sXMLDoc xml containing the tag
     */
    public void unsupportedTag(String sTagName, String sXMLDoc) {
        System.out.println("unsupported tag: " + sTagName);
    }

    /**
     * Message Received (Incoming Jabber Instant Message)
     * (a) If a chat window exists for sFrom send msg to current chat window.
     * (b) If a chat window doesn't exist open one with the new msg.
     * @param sFrom JID of who the iq is from
     * @param sType type of iq message
     * @param sSubject subject of message
     * @param sBody body of message
     * @param sThread thread message is related to
     */
    public void msgReceived(String sFrom, String sType, String sSubject, String sBody, String sThread) {
        System.out.println("sFrom:" + sFrom + " sType:" + sType + " sThread:" + sThread + " sSubject:" + sSubject + " sBody:" + sBody);
        //-----------------------------------------
        // Find sender in chat window vector
        //-----------------------------------------
        JID jidFrom = new JID(sFrom);        // create a JID from input param

        boolean bSystemMsg = false;
        //-----------------
        // groupchat
        //-----------------
        boolean bGroupChat = false;
        if (sType.equals("groupchat")) {
            bGroupChat = true;
            // Since it is a groupchat we need to set the nick, username to ""
            // Leave the Resource alone so Regular messages don't get confused
            // with Status Messages
            //jidFrom.setNick("");
            //jidFrom.setUsername("");

            // get nickname associated with this resource and set the nickname
            Object obj = groupChats.get(jidFrom.getUsername());
            if (obj == null) {
                //if (sSubject[x:3] == "invite:")
                // if this is an invite to a chat room prompt user to join it.
                if (sSubject.indexOf("invite:") == 0) {
                    groupChatPrompt(jidFrom.getUsername());
                    return;
                }
            } else //if (obj != null)
            {
                if (jidFrom.getResource().length() == 0)    // generic msg to entire chat room
                {
                    bSystemMsg = true;
                    jidFrom.setNick("");                    // like user status change
                } else {
                    if (((GroupChat)obj).getMyResource().equals(jidFrom.getResource()))
                        return;
                    jidFrom.setNick(((GroupChat)obj).getNick(jidFrom.getResource()));
                    System.out.println("Msg From Nick: " + jidFrom.getNick());
                }
            }
        }


        //-----------------------------------------
        // If we're not already chatn with this person
        // 1) Add user to list (if not already there)
        // 2) Start a chat window with this user
        //-----------------------------------------
        boolean bFoundChat = Chats.find(jidFrom);
        if (!bFoundChat) {
            //System.out.println("no chat found, start a new one");
            // 1st look into roster list, if not there either then add user
            JID jidFromList = zList.getJID(jidFrom, JID.MSK_USERNAME + JID.MSK_SERVER);
            if (jidFromList == null)                // existing roster member?
            {                                       // nope, create new user
                // user not found in roster listj, do nothing
                if (jidFrom.getUsername().length() == 0)
                    return;

                // if using groupchat as roster do nothing
                if (bUsingGroupChatAsRoster(jidFrom) == true)
                    return;

                zList.jidAdd(Config.FRIENDS, jidFrom);
                jidFromList = jidFrom;
            }                                       // end, create new user

            int nloc[] = new int[2];                // top/left
            childWindowPlacement( nloc, 200, 200 );
            if (bGroupChat) {
                // if not using groupchat as roster start chat window
                if (bUsingGroupChatAsRoster(jidFromList) == false)
                    Chats.startChatn(jidFromList.getJID(), jidMe, jidFromList, nloc[0], nloc[1], true);       // start chat window
            } else {
                Chats.startChatn(jidFromList.getNick(), jidMe, jidFromList, nloc[0], nloc[1]);       // start chat window
            }
        }

        //-----------------------------------------
        // Send msg to corresponding chat window
        //-----------------------------------------
        if (bSystemMsg)
            if (bGroupChat)
                Chats.systemMsg(jidFrom, moStringCleanup.Cleanup(sBody));
            else
                Chats.statusSet(jidFrom, moStringCleanup.Cleanup(sBody));
        else
            Chats.msgReceived(jidFrom, moStringCleanup.Cleanup(sBody));
    }                                               // end, msgReceived

    /**
    * A reply to your message has been started 
    * @param from JID of who is typing
    * @param msg message indicatin replying
    */
    /*
    public void msgReplyStarted(String from, String msg)
    {
        JID jidFrom = new JID(from);        // create a JID from input param
    Chats.msgReplyStarted(jidFrom, msg);	
    }
    */
    /**
     * Presence Received (Incoming Jabber Presence)
     * Presence packet received from the jabber server:
     * a) add/update the roster list accordingly
     * b) handle subscription request(s) if any
     * Each packet has a corresponding type: available, unavailable, subscribe, unsubscribe,,,.
     * There is also a corresponding status input/display.  But I think all of this can be ignored if there's an error code.
     * @param sTo The JID of who the presence is bound for.
     * @param sFrom The JID of who the presence is from. The server puts this attribute into presences to prevent spoofing.
     * @param sType Describes the type of presence so that different GUI styles may be supported by the clients. 
     * (available, unavailable,,,)
     * @param sStatus availability message, Used in conjunction with show to give a custom description of a users availability.
     * @param sPriority priority -- the ability to rank a user's presence
     * @param sShow Tells client applications how to display a user's online presence using a fixed set of options. The available options represent "away from the computer," "free for chat," "away from the computer for an extended period," and "do not disturb."
     * @param sError ?
     * @param sCode  ?
     */
    public void presenceReceived(String sTo, String sFrom, String sType, String sStatus,
                                 String sPriority, String sShow, String sError, String sCode) {
        System.out.println("presenceReceived:To:" + sTo + " From:" + sFrom + " Type:" + sType + " Status:" + sStatus + " Show:" + sShow + " Code:" + sCode);
        if (sCode.equals("401"))
            return;

        JID jidFrom = new JID(sFrom);
        if (jidFrom.getJID().toString().equals(jidMe.getJID().toString()))
            return;             // ignore myself.

        Object objGroupChatRoom = groupChats.get(jidFrom.getUsername());
        //------------------------------------------------
        // Subscribe to someone.
        // Display dialog for input of subscribee.
        //------------------------------------------------
        if (sType.equals("subscribe")) {
            JID jidFromList = zList.getJID(jidFrom, JID.MSK_USERNAME + JID.MSK_SERVER);

            // User already in list?  Get group JID is in.
            if (jidFromList != null)        // initiated from me
            {
                // this one is initiated (I think) from me subscribing to you.
                // if we display the msg again this would be a dupe, so don't do it!!!
                jidFromList.setGroup(zList.getGroupName(jidFromList, JID.MSK_USERNAME + JID.MSK_SERVER));
                rosterAddUser(jidFromList);     //automatically add so JabberIM will remove from <pending> list
                return;
            } else {
                jidFromList = jidFrom;      // initiated from outside
            }
            // Show New User dialog
            InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Add Contact");
            oInput.addQuestion("Add contact to your list","JID", jidFromList.getJID(), false, false);
            oInput.addQuestion("Nickname: (this name will appear in your contact list)", "Nick", jidFromList.getNick());
            String sGroup = jidFromList.getGroup();
            if (sGroup.length() == 0)
                sGroup = Config.FRIENDS;
            oInput.addQuestion("Group to add this contact to:", "Group", sGroup);
            oInput.addButton("Accept", "rosterAddUser");
            oInput.setDefaultButton("Accept");
            oInput.addButton("Decline", "declineSubscription");

            int nloc[] = new int[2];                // top/left
            childWindowPlacement( nloc, 240, 220 ); // compute next child loc
            oInput.setLocation(nloc[0], nloc[1]);
            oInput.inputbox();
        }

        //------------------------------------------------
        // Someone is unsubscribing from user.
        // Expect an 'unsubscribed', place user in unsubscribers
        // list so when get unsubscribed no msg displayed.
        //------------------------------------------------
        else if (sType.equals("unsubscribe")) {
            JID jidFromList = zList.getJID(jidFrom, JID.MSK_USERNAME + JID.MSK_SERVER);
            if (jidFromList != null) {
                vUnsubscribers.addElement(jidFrom.getUsername() + "@" + jidFrom.getServer());
                msgDialogLaunch( "User Unsubscribe", jidFromList.getUsername() + "@" + jidFromList.getServer() + "\n"+"is unsubscribing from your presence information"+"\n"+"and is being dropped from your contact list." );
            }
        }

        //------------------------------------------------
        // Someone has 'unsubscribed' from us.
        // if user B is in unsubscribe list
        //    (a) do nothing, User B initiated unsubscribe and msg has already been displayed
        // else
        //    (a) User B rejected subscribe or
        //    (b) User B doesn't exist or
        //    (c) User B server is down
        //------------------------------------------------
        else if (sType.equals("unsubscribed")) {
            JID jidFromList = zList.getJID(jidFrom, JID.MSK_USERNAME + JID.MSK_SERVER);
            if (vUnsubscribers.contains(jidFrom.getJID()) == true) {                                       // remove user from unsubscribe list, msg already displayed
                vUnsubscribers.removeElement(jidFrom.getJID());
            } else {

                String nl = "\n";
                if (sStatus.equals("Not Found"))
                    msgDialogLaunch( "User not found",
                                     "We were unable to add " + jidFrom.getJID() +
                                     " to your contact list."+nl+
                                     "Possible reasons:"+nl+
                                     "  (1) We were unable to contact the users "+
                                     "Jabber server."+nl+
                                     "  (2) The Jabber address you supplied "+
                                     "does not exist." );
                else
                    msgDialogLaunch( "Subscription Denied", jidFromList.getJID() + " denied your subscription request." );
            }

            // Remove user from roster display and redisplay roster
            if (zList.jidRemove(jidFrom,  JID.MSK_USERNAME + JID.MSK_SERVER) == true)
                ;
        }

        // groupchat user leaving a room
        else if ((objGroupChatRoom != null) && (sType.equals("unavailable"))) {
            ((GroupChat)objGroupChatRoom).userLeave(jidFrom.getResource());
            jidFrom.setGroup(jidFrom.getJID());
            if (zList.jidRemove(jidFrom, JID.MSK_GROUP + JID.MSK_RESOURCE) == false)
                ;
        }
        // not a group chat, somone now unavailable.
        else if( (sType.equals("unavailable")) ||
                 (sType.equals("error")) ) {
            // inform chat window of status change
            sStatus = moStringCleanup.Cleanup(sStatus);
            Chats.statusSet(jidFrom, sStatus);

            // set status to "".
            zList.jidUpdate(jidFrom, "", "unavailable",
                            JID.MSK_USERNAME + JID.MSK_SERVER);
        }
        //------------------------------------------------
        // Normal user, no change required.
        //------------------------------------------------
        else {
            // Handle special case for transports
            // if transport detected, update name to match roster
            if (jidFrom.getUsername().length() == 0) {
                if (agents.containsKey(jidFrom.getServer())) {
                    jidFrom.setUsername((String)agents.get(jidFrom.getServer()));
                }
            }

            // We can get presences with an empty state that we have to
            // process. E.g. PSI does this for the online presence by default.
            // Therefore we can't just ignore them.
            //
            // if nothing in type or status don't update display (unless user is now unavailable)
            /*            if ((objGroupChatRoom == null) && ((sStatus == null) || (sStatus.length() == 0)))
                        {
                                ; // don't update display, possibly JabberIM sending a started typing msg
                        }    
                            
                        // Update status in roster list
                        else 
                        {                                           // status change
            */
            if (sStatus == null)
                sStatus = "";
            if (sStatus.equals(""))
                sStatus = "Online";
            //if (sStatus.equalsIgnoreCase("disconnected"))
            //    sStatus = "";

            // inform chat window of status change
            sStatus = moStringCleanup.Cleanup(sStatus);

            // update roster display
            if (objGroupChatRoom != null)    // groupchat user?
            {
                jidFrom.setGroup(jidFrom.getJID());  // set group so future status updates will work
                // if not using groupchat as roster update roster
                if (bUsingGroupChatAsRoster(jidFrom) == false) {
                    jidFrom.setNick(((GroupChat)objGroupChatRoom).getNick(jidFrom.getResource()));
                    Chats.statusSet(jidFrom, sStatus);
                    if (zList.jidUpdate(jidFrom, sStatus, sShow, JID.MSK_GROUP + JID.MSK_RESOURCE) == false)
                        ; // nothing.
                }
            }
            else {
                Chats.statusSet(jidFrom, sStatus);
                if (zList.jidUpdate(jidFrom, sStatus, sShow, JID.MSK_USERNAME + JID.MSK_SERVER) == false)
                    ;
            }
            /*
                        }                                           // end, status change
            */
        }                                               // end, no errors?
    }                                                   // end, presenceReceived


    /**
    * Communications Closed
    */
    public void connectionClosed() {
        if (testConnect.testing() == true)
            connectionTest();
    }

    /**
    * Got stream id (needed for digest auth)
    */
    public void gotStreamID(String id) {
        streamID = id;
    }

    /**
    * Test connection with jabber server.  
    * If test fails try to reconnect/logon.
    */
    public void connectionTest() {
        if (moJabberComm.sendRawXML(" ") != Constants.SEND_SUCCEEDED) {
            System.out.println("sendRawXML for connectionTest() failed");
            testConnect.testing(false);
            logOff();
            logonUser( jidMe.getUsername(), txtPassword );
        }
    }

    /**
     * XML received from the Jabber Server, one character at a time
     * @param sXML Raw xml that was received
     */
    public void rawXML(String sXML) {}

    /**
     * Communications Error
     * 
     * @param sXML error xml string
     */
    public void error(String sXML) {
        System.out.println("XML error: " + sXML);
    }
    //==========^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=^=========
    //========= END Required iJabberClient methods
    //===================================================

    /**
     * Convert the roster list to XML
     */
    public String rosterListToXML() {
        return zList.toXML(jidMe.getNick());
    }

    /**
     * Invite user to groupchat.
     * @param chatRoom to invite user to
     * @param toJID user to invite
     * @param inviteMsg additional message to display with invite
     */
    public int inviteUser(String chatRoom, String toJID, String inviteMsg) {
        return moJabberComm.groupChatInvite(chatRoom, grpChatServer, toJID, inviteMsg);
    }

    /**
     * Convert the supplied XML (generated by InputDialog)
     * into a valid JID
     * @param XML verions of JID
     */
    private JID getJIDFromXML(String XML) {
        MiniXMLParser oParser = new MiniXMLParser();
        String sJID = oParser.parseTag(XML, "JID");
        String sNick = oParser.parseTag(XML, "Nick");
        String sGroup = oParser.parseTag(XML, "Group");
        JID oJID = new JID(sJID, sNick, sGroup, true);
        return oJID;
    }

    /**
     * Is debug on?
     */
    public boolean isDebug() {
        return bDebug;
    }

    /**
     * Am I logged off?
     */
    public boolean isLoggedOff() {
        String s = jidMe.getStatus();
        return ( s != null && s.equals(Config.LOGGED_OFF) );
    }

    /**
     * Are we groupchatting??
     */
    public boolean isGroupChatting() {
        if (groupChats.isEmpty()) {
            return false; // No GroupChats
        } else {
            return true; // We have GroupChats Open
        }
    }

    /**
     * Create the Advanced Settings Panel
     */
    public void advancedSettings() {
        // Open a Dialog box with a Text box asking
        // for a conferencing server.
        InputDialog oInput = new InputDialog(this, clientColor, Config.CLIENT_FONT, "Advanced Settings");
        oInput.addQuestion("Conferencing Server: ", "Server", grpChatServer, true, false,":@ \"<>\'&");
        oInput.addButton("OK", "setGroupChatServer");
        oInput.setDefaultButton("OK");
        oInput.addButton("Cancel");

        int nloc[] = new int[2];         // top/left
        childWindowPlacement( nloc, 240, 250 );
        oInput.setLocation(nloc[0], nloc[1]);
        oInput.inputbox();
    }

    /**
     * Check for valid input and set grpChatServer
     */
    public void setGroupChatServer(String XML) {
        // Set grpChatServer to what is passed in.
        // If what is passed in isn't valid then stick
        // with what we already have set.
        MiniXMLParser oParser = new MiniXMLParser();
        String groupChatServer = oParser.parseTag(XML, "Server");
        if (groupChatServer.length() > 0) {
            System.out.println("New GroupChatServer: " + groupChatServer);
            setGrpChatServer(groupChatServer);
        } else {
            return;
        }
    }

    /**
     * Set grpChatServer
     * @param newGrpChatServer
     */
    public void setGrpChatServer(String newGrpChatServer) {
        grpChatServer = newGrpChatServer;
    }
}
// end, JabberApplet class


/* bischoff */

class WindowControl implements WindowListener {
  public void windowClosing (WindowEvent e) {
    //a.stop();
    System.exit(0);
  }
  public void windowClosed (WindowEvent e) { }
  public void windowOpened (WindowEvent e) { }
  public void windowIconified (WindowEvent e) { }
  public void windowDeiconified (WindowEvent e) { }
  public void windowActivated (WindowEvent e) { }
  public void windowDeactivated (WindowEvent e) { }
}
/* bis hier */