package org.jabber.applet.connection;

import java.net.*;
import java.io.*;
import java.util.*;
/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
* This class sends and receives data packets using Input/Output Streams
* 
* @author	Jonathan Paulson
* @version 	$ 0.7.0 $
*/
public abstract class Connection extends Thread {
    protected IConnectionClient moConnectionClient = null;
    protected ConnectionState moConnState = null;

    /**
     * Used to communicate back to the object
     * 
     * @param connectionClient	Reference to the Object that implements IConnectionClient  	
     */
    public Connection(IConnectionClient connectionClient) {
        moConnState = new ConnectionState(ConnectionState.CLOSED);
        moConnectionClient = connectionClient;
    }

    /**
     * Sends data packets to Server
     * 
     * @param str      Text to send to Server 
     * @return	      True if it was able to send Data, False otherwise 
     */
    public abstract boolean send(String str);

    /**
     * Connect to specified Server/Port
     * 
     * @param host      Host Name or IP Address 
     * @param port      Port to connect to 
     * @return	       True Connection made, False otherwise 		
     */
    public abstract ConnectionState connect(String host, int port);

    /**
     * Disconnect from host
     * 
     */
    public abstract void disconnect();

    /**
     * Pass on data that was received to the IConnectionClient
     *
     * @param str Data received
     */
    protected void dataReceived(String str) {
        //System.out.println("RECV: "+str);
        if (moConnectionClient != null) {
            moConnectionClient.dataReceived(str);
        }
    }

    /**
     * Send communications error message to the IConnectionClient
     *
     * @param str Error message
     */
    protected void error(String str) {
        if (moConnectionClient != null) {
            moConnectionClient.error(str);
        }
    }

    /**
     * Prepare the instance for beeing removed
     */
    public abstract void cleanup();
}
