package org.jabber.applet.tools;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/


/**
* This class handles only simple XML parsing (meant to be small)
* 
* @author	Jonathan Paulson
* @version 	$ 0.9.0 $
*/
public class MiniXMLParser {
    private int				miMode					= 0;
    private int				miStart					= 0;

    /**
    * Search String for a Tag (a Tag is defined by <>) 
    * or an Attribute (value within a tag ie <Tag value='zzz'>) for the 
    * FIRST occurance of a tag (ie this will only to find the first 
    * tag if it's a repeating tag)
    *
    * @param XML	    XML String	
    * @param tag		Tag name 
    * @return The contents of the specified Tag	
    */
    public String parseTag(String XML, String tag) {
        //Reset starting position to search for
        miStart = 0;
        return parseNextTag(XML, tag, false);
    }

    public String parseTag(String XML, String tag, boolean includeTags) {
        //Reset starting position to search for
        miStart = 0;
        return parseNextTag(XML, tag, includeTags);
    }

    /**
    * Search String for a Tag (a Tag is defined by <>) 
    * or an Attribute (value within a tag ie <Tag value='zzz'>).
    * This method will pickup where the last search left off (for repeating tags)
    * Call parseTag("","") to reset the index of last search;
    *
    * @param XML			XML String
    * @param tag			Tag name
    * @param includeTag		If True, Return the Begin & End Tags too (ie "<Item ...>...</item>")
    *						This is usefull for Tags that contain Attributes and Values 
    * @return				The contents of the specified Tag (String)	
    */
    public String parseNextTag(String XML, String tag) {
        return parseNextTag(XML, tag, false);
    }
    public String parseNextTag(String XML, String tag, boolean includeTags) {
        String	sParseTag = "";     //Default not found
        String  sLowerCaseXML = XML.toLowerCase();
        int		iStart;
        int		iEnd;
        int		iFromIndex;

        if (miStart < 0 || miStart > sLowerCaseXML.length())
            miStart = 0;

        if (sLowerCaseXML.length() == 0)
            return "";

        iFromIndex = miStart;
        //Reset (This is set down below if successful)
        miStart = 0;

        //convert to lower case for comparisons
        tag = tag.toLowerCase();

        //first checke to see if it's an empty tag ie... <tagname/>
        if (sLowerCaseXML.indexOf("<" + tag + "/>", iFromIndex) != -1) {
            if (includeTags)
                return "<" + tag + "/>";
            else
                return "";
        }

        //now look for begin tag
        iStart = sLowerCaseXML.indexOf("<" + tag + ">", iFromIndex);

        if (iStart == -1) {
            //Check to see if this tag has values ie... <presence to="Savannah" ...>
            iStart = sLowerCaseXML.indexOf("<" + tag + " ", iFromIndex);
            if (iStart != -1) {
                //Find Tags end bracket
                iEnd =  sLowerCaseXML.indexOf(">", iStart+1);

                if (iEnd != -1) {
                    //We now want to see if the tag is empty (ie <presence to="Savannah" .../>)
                    if (sLowerCaseXML.charAt(iEnd -1) == '/') {
                        //This is an empty tag
                        miStart = iEnd + 1;
                        try {
                            if (includeTags)
                                return XML.substring(iStart, iEnd+1);
                            else
                                return "";
                        } catch(StringIndexOutOfBoundsException e) {
                            return "";
                        }
                    } else {
                        //Not an empty tag
                        if (!includeTags)
                            //Move past tag because we only want the tag contents.
                            iStart =iEnd + 1;
                    }

                }

            }
        } else {
            //move start to first position past closing bracket >
            //we only want the tag contents
            if (!includeTags)
                iStart = iStart + tag.length() + 2;
        }

        if (iStart != -1) {
            //look for end tag
            String sEndTag = "</" + tag + ">";
            iEnd = sLowerCaseXML.indexOf(sEndTag, iFromIndex);

            if (iEnd != -1) {
                //Extract contents of tag
                try {
                    if (includeTags)
                        sParseTag = XML.substring(iStart,iEnd + sEndTag.length());
                    else
                        sParseTag = XML.substring(iStart,iEnd);
                } catch(StringIndexOutOfBoundsException e) {
                    sParseTag = "";
                }
                miStart = iEnd + 1;
            } else {
                //This is a tag, with attributes, that is empty  ie <Tag value='zzz' />
                try {
                    if (includeTags)
                        sParseTag = XML.substring(iStart, XML.indexOf("/>", iStart + 1) + 2);
                    else
                        //contents empty
                        sParseTag = "";
                } catch(StringIndexOutOfBoundsException e) {
                    sParseTag = "";
                }

            }
        } else {
            //Check to see if it's an Attribute (value within a tag ie <Tag value='zzz'>)
            iStart = sLowerCaseXML.indexOf(tag + "=", iFromIndex);
            if (iStart != -1) {
                //move start past Attribute and equal sign
                iStart += tag.length() + 1;
                //You can either use a single or double quote (' or ") so
                //determine which one it is so that you can search for ending char
                String sSurroundingChar = sLowerCaseXML.substring(iStart, iStart + 1);
                iEnd = sLowerCaseXML.indexOf(sSurroundingChar, iStart + 1);

                if (iEnd != -1) {
                    //Extract contents of tag
                    iStart += 1;
                    try {
                        sParseTag = XML.substring(iStart, iEnd);
                    } catch(StringIndexOutOfBoundsException e) {
                        sParseTag = "";
                    }
                    miStart = iEnd + 1;
                }
            }
        }
        return sParseTag;
    }
}
