package org.jabber.applet.tools;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
* This class converts special characters to XML safe and vice versa
* 
* @author	David Scott
*/
public class StringCleanup {
    public StringCleanup() {}

    /**
     * Cleanup string for display or for XML transmission.  
     * Replace all known chars to cause XML problems with their XML safe equivalents
     * or replace the XML safe codes with their ASCII equivalents when displaying to users
     * depending on the value of decode.
     *
    * Example.  The first line will be converted to the second line if decode is true
    *	 &lt;TheJonz&gt; said &quot;It&apos;s over here&quot;
    *   <TheJonz> said "it's over here"
     * 
     * @param str		string to be cleaned up
     * @param decode	true if you are converting FROM XML, false if converting TO XML
     * @return			the result of the conversion
     */
    public String Cleanup(String str) {
        return Cleanup(str, true);
    }

    public String Cleanup(String str, boolean decode) {
        String saXMLEquivalent[] = {"&amp;", "&apos;", "&quot;", "&lt;", "&gt;"};
        String saSpecialChars[] = {"&", "\'", "\"", "<", ">"};
        String sFind;
        String sReplace;
        boolean bFound;
        int iPos = -1;
        int i = 0;

        while (i < saXMLEquivalent.length) {
            String newStr = "";
            if (decode) {
                //Search for XML encodeded string and convert it back to plain ASCII
                sFind = saXMLEquivalent[i];
                sReplace = saSpecialChars[i];
            } else {
                //Search for special chars in ASCII and replace with XML safe chars
                sFind = saSpecialChars[i];
                sReplace = saXMLEquivalent[i];
            }
            do {
                iPos = str.indexOf(sFind, ++iPos);
                if (iPos > -1) {
                    newStr = newStr + str.substring(0, iPos) + sReplace + str.substring(iPos+sFind.length(),str.length());
                    str = newStr;
                    newStr = "";
                    bFound = true;
                } else {
                    bFound = false;
                }
            } while ( bFound );
            i++;
        }
        return(str);
    }
}
































